-- ===================================================================
-- "default" mod for MeseLite Game

WATER_ALPHA = 160
WATER_VISC  =   1
LAVA_VISC   =   7

default = {}
default.LIGHT_MAX = 14

local   modname   = minetest.get_current_modname()
local   modpath   = minetest.get_modpath (modname)
default.modpath   = modpath

-- ===================================================================

-- This variable is  set to true for  Maksym 2 and to false for  Final
-- MT 4.

default.ismc = false

if  (rawget (_G, "nodeupdate") == nil) or
    (nodeupdate == nil) then
    minetest.log ("action", "nodeupdate isn't present")

    nodeupdate =  function (p)
        core.check_for_falling (p)
    end
    default.ismc = true

    -- 210712 RJK: Fix a missing-biomes bug
    local mg_flags = minetest.get_mapgen_setting ("mg_flags")
    if    mg_flags:find "biomes" == nil then
          mg_flags = mg_flags .. ",biomes"
          minetest.set_mapgen_setting ("mg_flags", mg_flags, true)
    end
else
    minetest.log ("action", "nodeupdate is present")
end

-- ===================================================================

if  (rawget (_G, "nodeupdate_single") == nil) or
    (nodeupdate_single == nil) then
    minetest.log ("action", "nodeupdate_single isn't present")

    nodeupdate_single =  function (p)
        core.check_single_for_falling (p)
    end
else
    minetest.log ("action", "nodeupdate_single is present")
end

-- ===================================================================

dofile (modpath .. "/ocutil.lua"         )

-- ===================================================================

default.old_content_id = minetest.get_content_id

minetest.get_content_id = function (s)
    if (ocutil.node_missing (s)) then return nil end
    return default.old_content_id (s)
end

-- ===================================================================

default.old_register_entity = minetest.register_entity

minetest.register_entity = function (name, def)
    -- minetest.log ("action", "register_entity " .. name)

    if default.ismc then
        if  def.automatic_rotate == false or
            def.automatic_rotate == true  then
            def.automatic_rotate =  0
        end
    end

    default.old_register_entity (name, def)
end

-- ===================================================================

default.old_register_node = minetest.register_node

minetest.register_node = function (name, def)
    -- minetest.log ("action", "regnode " .. name)

    if default.ismc then
        local obj

        if def.drowning == false then def.drowning = 0 end
        if def.drowning == true  then def.drowning = 1 end

        obj = def.inventory_image
        if        obj  ~= nil      and
            type (obj) == 'table'  then
            def.inventory_image = def.inventory_image [0]
        end

        obj = def.use_texture_alpha
        if        obj  ~= nil      and
            type (obj) == 'string' and
                  obj  == 'blend'  then
            def.use_texture_alpha = true
        end

        obj = def.wield_image
        if        obj  ~= nil      and
            type (obj) == 'table'  then
            def.wield_image = def.wield_image [0]
        end
    end

    default.old_register_node (name, def)
end

-- ===================================================================

if  table.insert_all == nil then
    table.insert_all =  function (t, other)
	    for i=1, #other do
		    t [#t + 1] = other [i]
    	end
    	return t
    end
end

-- ===================================================================

local function safe_gsub (s, replace, with)
    local i1, i2 = s:find(replace, 1, true)
    if not i1 then return s, false end
    return s:sub(1, i1 - 1) .. with .. s:sub(i2 + 1), true
end

-- ===================================================================

-- Chat message formatter
-- Implemented in Lua to allow redefinition

core.format_chat_message = function (name, message)
    local error_str = "Invalid chat message format - missing %s"
    local str = core.settings:get("chat_message_format")
    local replaced

    -- Name
    str, replaced = safe_gsub(str, "@name", name)
    if not replaced then
        error(error_str:format("@name"), 2)
    end

    -- Timestamp
    str = safe_gsub(str, "@timestamp", os.date("%H:%M:%S", os.time()))

    -- Insert the message into the string only after finishing all other processing
    str, replaced = safe_gsub(str, "@message", message)
    if not replaced then
        error(error_str:format("@message"), 2)
    end

    return str
end

-- ===================================================================

local savereg = minetest.register_node

minetest.register_node = function (name, def)
--  minetest.log ("action", "register_node " .. name)

    if default.ismc then
        if def.drowning == false then def.drowning = 0 end
        if def.drowning == true  then def.drowning = 1 end

        if  def.inventory_image ~= nil and
            type (def.inventory_image) == 'table' then
            def.inventory_image = def.inventory_image [0]
        end

        if  def.wield_image ~= nil and
            type (def.wield_image) == 'table' then
            def.wield_image = def.wield_image [0]
        end
    end

    savereg (name, def)
end

-- ===================================================================

dofile (modpath .. "/mapgen.lua"         )
dofile (modpath .. "/sound_defaults.lua" )

-- ===================================================================
-- GUI-related.

default.gui_bg = "bgcolor[#080808BB;true]"
default.gui_bg_img = "background[5,5;1,1;gui_formbg.png;true]"
default.gui_slots = "listcolors[#00000069;#5A5A5A;#141318;#30434C;#FFF]"

function default.get_hotbar_bg (x, y)
    local out = ""
    for i=0,7,1 do
        out = out .."image["..x+i..","..y..";1,1;gui_hb_bg.png]"
    end
    return out
end

-- ===================================================================
-- Set a noticeable inventory formspec for players

minetest.register_on_joinplayer(function(player)
    local cb = function(player)
        local playname = player:get_player_name()
        minetest.chat_send_player (playname,
            "Welcome to Minetest Lite."   ..
            " This is a super-light subgame")
        minetest.chat_send_player (playname,
            "subgame originally by Theo." ..
            " It's small and fast but has many features.")
    end
    minetest.after(2.0, cb, player)
end)

-- ===================================================================
-- Tool definitions.

-- The hand
minetest.register_item (":", {
    type = "none",
    wield_image = "wieldhand.png",
    wield_scale = {x=1,y=1,z=2.5},
    tool_capabilities = {
        full_punch_interval = 1.0,
        max_drop_level = 0,
        groupcaps = {
            fleshy = {times= {[2]=2.00, [3]=1.00}, uses=0, maxlevel=1},
            crumbly = {times= {[2]=3.00, [3]=0.70}, uses=0, maxlevel=1},
            snappy = {times= {[3]=0.40}, uses=0, maxlevel=1},
            oddly_breakable_by_hand = {times= {[1]=7.00,[2]=4.00,[3]=1.40}, uses=0, maxlevel=3},
        },
        damage_groups = {fleshy=1},
    }
})

-- ===================================================================
-- Picks

minetest.register_tool ("default:pick_steel", {
    description = "Steel Pickaxe",
    inventory_image = "default_tool_steelpick.png",
    tool_capabilities = {
        max_drop_level=1,
        groupcaps= {
            cracky= {times= {[1]=4.00, [2]=1.60, [3]=1.00}, uses=10, maxlevel=2}
        },
        damage_groups = {fleshy=4},
    },
})

-- ===================================================================
-- Support old code

function default.spawn_falling_node(p, nodename)
    spawn_falling_node(p, nodename)
end

-- Horrible crap to support old code
-- Don't use this and never do what this does, it's completely wrong!
-- (More specifically, the client and the C++ code doesn't get the group)

function default.register_falling_node(nodename, texture)
    minetest.log("error", debug.traceback())
    if minetest.registered_nodes[nodename] then
        minetest.registered_nodes[nodename].groups.falling_node = 1
    end
end

-- ===================================================================
-- Global callbacks

if false then
function on_step (dtime)
end
minetest.register_globalstep (on_step)

function on_placenode (p, node)
end
minetest.register_on_placenode (on_placenode)

function on_dignode (p, node)
end
minetest.register_on_dignode (on_dignode)

function on_punchnode (p, node)
end
minetest.register_on_punchnode (on_punchnode)
end

-- ===================================================================

dofile (modpath .. "/obj_books.lua"       )
dofile (modpath .. "/obj_chests.lua"      )
dofile (modpath .. "/obj_ground.lua"      )
dofile (modpath .. "/obj_ladders.lua"     )
dofile (modpath .. "/obj_lava.lua"        )
dofile (modpath .. "/obj_plants.lua"      )
dofile (modpath .. "/obj_stone.lua"       )
dofile (modpath .. "/obj_trees.lua"       )
dofile (modpath .. "/obj_water.lua"       )
dofile (modpath .. "/obj_misc.lua"        )

dofile (modpath .. "/furnace.lua"         )
dofile (modpath .. "/furnace_locked.lua"  )
dofile (modpath .. "/player.lua"          )
dofile (modpath .. "/crafting.lua"        )

dofile (modpath .. "/nodesy.lua"          )
dofile (modpath .. "/outerspace.lua"      )
dofile (modpath .. "/convert.lua"         )

-- ===================================================================
-- End of file.
