default.chest_formspec =
    "size[8,9]"..
    default.gui_bg..
    default.gui_bg_img..
    default.gui_slots..
    "list[current_name;main;0,0.3;8,4;]"..
    "list[current_player;main;0,4.85;8,1;]"..
    "list[current_player;main;0,6.08;8,3;8]"..
    default.get_hotbar_bg(0,4.85)

function default.get_locked_chest_formspec(pos)
    local spos = pos.x .. "," .. pos.y .. "," ..pos.z
    local formspec =
        "size[8,9]"..
        default.gui_bg..
        default.gui_bg_img..
        default.gui_slots..
        "list[nodemeta:".. spos .. ";main;0,0.3;8,4;]"..
        "list[current_player;main;0,4.85;8,1;]"..
        "list[current_player;main;0,6.08;8,3;8]"..
        default.get_hotbar_bg(0,4.85)
 return formspec
end

minetest.register_node("default:chest", {
    description = "Chest",
    tiles = {
        "default_chest_top.jpg",
        "default_chest_top.jpg",
        "default_chest_side.jpg",
        "default_chest_side.jpg",
        "default_chest_side.jpg",
        "default_chest_front.jpg"
    } ,
    paramtype2 = "facedir",
    groups = {choppy=2,oddly_breakable_by_hand=2},
    legacy_facedir_simple = true,
    is_ground_content = false,
    sounds = default.sound_wood(),
    on_construct = function(pos)
        local meta = minetest.get_meta(pos)
        meta:set_string("formspec",default.chest_formspec)
        meta:set_string("infotext", "Chest")
        local inv = meta:get_inventory()
        inv:set_size("main", 8*4)
    end,
    can_dig = function(pos,player)
        local meta = minetest.get_meta(pos);
        local inv = meta:get_inventory()
        return inv:is_empty("main")
    end,
    on_metadata_inventory_move = function(pos, from_list, from_index, to_list, to_index, count, player)
        minetest.log("action", player:get_player_name()..
                " moves stuff in chest at "..minetest.pos_to_string(pos))
    end,
    on_metadata_inventory_put = function(pos, listname, index, stack, player)
        minetest.log("action", player:get_player_name()..
                " moves stuff to chest at "..minetest.pos_to_string(pos))
    end,
    on_metadata_inventory_take = function(pos, listname, index, stack, player)
        minetest.log("action", player:get_player_name()..
                " takes stuff from chest at "..minetest.pos_to_string(pos))
    end,
})

local function has_locked_chest_privilege (meta, player)
    local plname = player:get_player_name()

    if minetest.check_player_privs (plname, "protection_bypass") then
        return true
    end
    if plname ~= meta:get_string("owner") then
        return false
    end
    return true
end

minetest.register_node("default:chest_locked", {
    description = "Locked Chest",
    tiles = {"default_chest_top.jpg", "default_chest_top.jpg", "default_chest_side.jpg",
        "default_chest_side.jpg", "default_chest_side.jpg", "default_chest_lock.jpg"},
    paramtype2 = "facedir",
    groups = {choppy=2,oddly_breakable_by_hand=2},
    legacy_facedir_simple = true,
    is_ground_content = false,
    sounds = default.sound_wood(),
    after_place_node = function(pos, placer)
        local meta = minetest.get_meta(pos)
        meta:set_string("owner", placer:get_player_name() or "")
        meta:set_string("infotext", "Locked Chest (owned by "..
                meta:get_string("owner")..")")
    end,
    on_construct = function(pos)
        local meta = minetest.get_meta(pos)
        meta:set_string("infotext", "Locked Chest")
        meta:set_string("owner", "")
        local inv = meta:get_inventory()
        inv:set_size("main", 8*4)
    end,
    can_dig = function(pos,player)
        local meta = minetest.get_meta(pos);
        local inv = meta:get_inventory()
        return inv:is_empty("main") and has_locked_chest_privilege(meta, player)
    end,
    allow_metadata_inventory_move = function(pos, from_list, from_index, to_list, to_index, count, player)
        local meta = minetest.get_meta(pos)
        if not has_locked_chest_privilege(meta, player) then
            return 0
        end
        return count
    end,
    allow_metadata_inventory_put = function(pos, listname, index, stack, player)
        local meta = minetest.get_meta(pos)
        if not has_locked_chest_privilege(meta, player) then
            return 0
        end
        return stack:get_count()
    end,
    allow_metadata_inventory_take = function(pos, listname, index, stack, player)
        local meta = minetest.get_meta(pos)
        if not has_locked_chest_privilege(meta, player) then
            return 0
        end
        return stack:get_count()
    end,
    on_metadata_inventory_put = function(pos, listname, index, stack, player)
        minetest.log("action", player:get_player_name()..
                " moves stuff to locked chest at "..minetest.pos_to_string(pos))
    end,
    on_metadata_inventory_take = function(pos, listname, index, stack, player)
        minetest.log("action", player:get_player_name()..
                " takes stuff from locked chest at "..minetest.pos_to_string(pos))
    end,
    on_rightclick = function(pos, node, clicker)
        local meta = minetest.get_meta(pos)
        if has_locked_chest_privilege(meta, clicker) then
            minetest.show_formspec(
                clicker:get_player_name(),
                "default:chest_locked",
                default.get_locked_chest_formspec(pos)
            )
        end
    end,
})

minetest.register_abm ({
    nodenames = { "default:chest" },
    interval  = 1.0   ,
    chance    = 1     ,
    catch_up  = false ,

    action = function(pos, node, active_object_count, active_object_count_wider)
        local meta = minetest.get_meta (pos)

        if meta == nil or
           meta:get_string ("formspec") == "" or
           meta:get_string ("formspec") == nil then
            meta:set_string("formspec",default.chest_formspec)
            meta:set_string("infotext", "Chest")
            local inv = meta:get_inventory()
            inv:set_size("main", 8*4)
        end
    end,
})
