-- ===================================================================

local dwg_tiles = {
    "default_grass.png" , "default_dirt.png" ,
    { name = "default_dirt.png^default_grass_side.png" ,
      tileable_vertical = false }
}

-- ===================================================================

ocutil.register_node_bucketable ("default:dirt_with_grass", {
    description = "Dirt with Grass" ,
    bucket_base = "dirt_with_grass" ,
    paramtype = "light",
    light_source  = 8,
    tiles       = dwg_tiles ,
    groups      = { crumbly = 3, soil = 1 } ,
    drop        = 'default:dirt_with_grass',

    sounds      = default.sound_dirt ({
        footstep = {name = "default_grass_footstep", gain = 0.25} ,
    }) ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:dirt", {
    description = "Dirt" ,
    bucket_base = "dirt" ,
    paramtype = "light",
    light_source  = 8,
    tiles = {"default_dirt.png"} ,
    groups = {crumbly = 3, soil = 1} ,
    sounds = default.sound_dirt() ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:sand", {
    bucket_base = "sand"                                ,
    description = "Sand"                                ,
    tiles       = { "default_sand.png" }                ,
    groups      = { crumbly=3, falling_node=1, sand=1 } ,
    sounds      = default.sound_sand()    ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:silver_sand", {

    bucket_base = "silver_sand"                         ,
    description = "Silver Sand"                         ,
    tiles       = { "default_silver_sand.png" }         ,
    groups      = { crumbly=3, falling_node=1, sand=1 } ,
    sounds      = default.sound_sand()                  ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:gravel", {

    bucket_base = "gravel"                             ,
    description = "Gravel"                             ,
    tiles       = { "default_gravel.png" }             ,
    groups      = { crumbly=2, falling_node=1 }        ,
--  sounds      = default.sound_gravel()               ,

    drop = {
        max_items = 1,
        items = {
--          {items = {'default:flint' } , rarity = 16} ,
            {items = {'default:gravel'}}
        }
    }
})

-- ===================================================================

minetest.register_node ("default:clay", {
    description = "Clay",
    tiles = { "default_clay.png" } ,
    groups = {crumbly=3},
    sounds = default.sound_dirt({
        footstep = "",
    }),
})

-- ===================================================================

-- 'is ground content = false' to avoid tunnels in sea ice or ice
-- rivers

ocutil.register_node_bucketable ("default:ice", {
    description = "Ice" ,
    bucket_base = "ice" ,
    is_ground_content = false,
    paramtype = "light",
    tiles     = { "default_ice.png" } ,

    groups = {
        cracky=3     , puts_out_fire=1  ,
        cools_lava=1 , slippery=3       ,
    } ,

    sounds = default.sound_glass() ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:snow", {
    bucket_base = "snow" ,
    description = "Snow" ,
    tiles = {"default_snow.png"} ,
    inventory_image = "default_snowball.png",
    wield_image = "default_snowball.png",
    paramtype = "light",
    buildable_to = true,
    floodable = true,
    drawtype = "nodebox",

    node_box = {
        type = "fixed",
        fixed = {
            {-0.5, -0.5, -0.5, 0.5, -0.25, 0.5} ,
        } ,
    } ,

    groups = {crumbly = 3, falling_node = 1, puts_out_fire = 1} ,
    sounds = default.sound_dirt({
        footstep = {name = "default_snow_footstep", gain = 0.15} ,
        dug = {name = "default_snow_footstep", gain = 0.2} ,
        dig = {name = "default_snow_footstep", gain = 0.2}
    }) ,

    on_construct = function (pos)
        pos.y = pos.y - 1
        if  minetest.get_node(pos).name ==
                "default:dirt_with_grass" then
            minetest.set_node (pos,
                { name = "default:dirt_with_snow" })
        end
    end ,
})

-- ===================================================================

ocutil.register_node_bucketable ("default:snowblock", {

    bucket_base = "snow_block" ,
    description = "Snow Block" ,
    tiles = {"default_snow.png"} ,
    groups = {crumbly = 3, puts_out_fire = 1} ,
    sounds = default.sound_dirt({
        footstep = {name = "default_snow_footstep", gain = 0.15} ,
        dug = {name = "default_snow_footstep", gain = 0.2} ,
        dig = {name = "default_snow_footstep", gain = 0.2}
    }),
})

-- ===================================================================

ocutil.register_node_bucketable ("default:dirt_with_snow", {
    bucket_base = "dirt_with_snow" ,
    description = "Dirt with Snow" ,
    tiles       = {
        "default_snow.png" ,
        "default_dirt.png" ,
        {
            name = "default_dirt.png^default_snow_side.png" ,
            tileable_vertical = false
        }
    } ,

    groups = {crumbly = 3, soil = 1} ,
    drop = 'default:dirt',
    sounds = default.sound_dirt({
        footstep = {name = "default_snow_footstep", gain = 0.15} ,
    }),
})

-- ===================================================================

minetest.register_abm ({
    nodenames = { "default:dirt" } ,
    interval = 2,
    chance = 200,
    action = function(pos, node)
        local above = {x=pos.x, y=pos.y+1, z=pos.z}
        local name = minetest.get_node(above).name
        local nodedef = minetest.registered_nodes[name]
        if nodedef and (nodedef.sunlight_propagates or nodedef.paramtype == "light")
                and nodedef.liquidtype == "none"
                and (minetest.get_node_light(above) or 0) >= 13 then
            if name == "default:snow" or name == "default:snowblock" then
                minetest.set_node(pos, {name = "default:dirt_with_snow"})
            else
                minetest.set_node(pos, {name = "default:dirt_with_grass"})
            end
        end
    end
})

-- ===================================================================

minetest.register_abm ({
    nodenames = { "default:dirt_with_grass" } ,
    interval = 2,
    chance = 20,
    action = function(pos, node)
        local above = {x=pos.x, y=pos.y+1, z=pos.z}
        local name = minetest.get_node(above).name
        local nodedef = minetest.registered_nodes[name]
        if name ~= "ignore" and nodedef
                and not ((nodedef.sunlight_propagates or nodedef.paramtype == "light")
                and nodedef.liquidtype == "none") then
            minetest.set_node(pos, {name = "default:dirt"})
        end
    end
})

-- ===================================================================
-- End of file.
