-- ===================================================================

minetest.register_node ("default:tree", {
    description = "Tree",
    tiles = { "default_tree_top.png", "default_tree_top.png", "default_tree.png" } ,
    is_ground_content = false,
    groups = {snappy=2,choppy=2,oddly_breakable_by_hand=1,wood=1},
    sounds = default.sound_wood(),
})

-- ===================================================================

minetest.register_node ("default:leaves", {
    description = "Leaves",
    drawtype = "allfaces_optional",
    visual_scale = 1.3,
    tiles = { "default_leaves.png" } ,
    paramtype = "light",
    is_ground_content = false,
    groups = {snappy=3},
    drop = {
        max_items = 1,
        items = {
            {
                -- player will get sapling with 1/20 chance
                items = {'default:sapling'},
                rarity = 20,
            },
            {
                -- player will get leaves only if he get no saplings,
                -- this is because max_items is 1
                items = {'default:leaves'},
            }
        }
    },
    sounds = default.sound_leaves(),
})

-- ===================================================================

minetest.register_node ("default:wood", {
    description = "Wood",
    tiles = { "default_wood.png" } ,
    is_ground_content = false,
    groups = {snappy=2,choppy=2,oddly_breakable_by_hand=2,wood=1},
    sounds = default.sound_wood(),
})

-- ===================================================================

minetest.register_node ("default:sapling", {
    description = "Sapling",
    drawtype = "plantlike",
    visual_scale = 1.0,
    tiles = { "default_sapling.png" } ,
    inventory_image = "default_sapling.png",
    wield_image = "default_sapling.png",
    paramtype = "light",
    walkable = false,
    groups = {snappy=2,dig_immediate=3,attached_node=1},
    sounds = default.node_sound_defaults(),
})

-- ===================================================================

minetest.register_node ("default:apple", {
    description = "Apple",
    drawtype = "plantlike",
    visual_scale = 1.0,
    tiles = { "default_apple.png" } ,
    inventory_image = "default_apple.png",
    paramtype = "light",
    is_ground_content = false,
    sunlight_propagates = true,
    walkable = false,
    groups = {fleshy=3,dig_immediate=3},
    on_use = minetest.item_eat(4),
    sounds = default.node_sound_defaults(),
})

-- ===================================================================
-- Grow tree function
--

local c_air    = minetest.get_content_id("air")
local c_ignore = minetest.get_content_id("ignore")
local c_tree   = minetest.get_content_id("default:tree")
local c_leaves = minetest.get_content_id("default:leaves")
local c_apple  = minetest.get_content_id("default:apple")

-- ===================================================================

function default.grow_tree(data, a, pos, is_apple_tree, seed)
    --[[
        NOTE: Tree-placing code is currently duplicated in the engine
        and in games that have saplings; both are deprecated but not
        replaced yet
    ]]--
    local pr = PseudoRandom(seed)
    local th = pr:next(4, 5)
    local x, y, z = pos.x, pos.y, pos.z
    for yy = y, y+th-1 do
        local vi = a:index(x, yy, z)
        if a:contains(x, yy, z) and (data[vi] == c_air or yy == y) then
            data[vi] = c_tree
        end
    end

    y = y+th-1 -- (x, y, z) is now last piece of trunk
    local leaves_a = VoxelArea:new{MinEdge= {x=-2, y=-1, z=-2}, MaxEdge= {x=2, y=2, z=2}}
    local leaves_buffer = {}

    -- Force leaves near the trunk
    local d = 1
    for xi = -d, d do
    for yi = -d, d do
    for zi = -d, d do
        leaves_buffer[leaves_a:index(xi, yi, zi)] = true
    end
    end
    end

    -- Add leaves randomly
    for iii = 1, 8 do
        local d = 1
        local xx = pr:next(leaves_a.MinEdge.x, leaves_a.MaxEdge.x - d)
        local yy = pr:next(leaves_a.MinEdge.y, leaves_a.MaxEdge.y - d)
        local zz = pr:next(leaves_a.MinEdge.z, leaves_a.MaxEdge.z - d)

        for xi = 0, d do
        for yi = 0, d do
        for zi = 0, d do
            leaves_buffer[leaves_a:index(xx+xi, yy+yi, zz+zi)] = true
        end
        end
        end
    end

    -- Add the leaves
    for xi = leaves_a.MinEdge.x, leaves_a.MaxEdge.x do
    for yi = leaves_a.MinEdge.y, leaves_a.MaxEdge.y do
    for zi = leaves_a.MinEdge.z, leaves_a.MaxEdge.z do
        if a:contains(x+xi, y+yi, z+zi) then
            local vi = a:index(x+xi, y+yi, z+zi)
            if data[vi] == c_air or data[vi] == c_ignore then
                if leaves_buffer[leaves_a:index(xi, yi, zi)] then
                    if is_apple_tree and pr:next(1, 100) <=  10 then
                        data[vi] = c_apple
                    else
                        data[vi] = c_leaves
                    end
                end
            end
        end
    end
    end
    end
end

-- ===================================================================
-- ABMs
--

minetest.register_abm ({
    nodenames = { "default:sapling" } ,
    interval = 10,
    chance = 50,
    action = function(pos, node)
        if minetest.get_item_group(minetest.get_node(
            {x = pos.x, y = pos.y - 1, z = pos.z}).name, "soil") == 0 then
            return
        end
        print("A sapling grows into a tree at "..minetest.pos_to_string(pos))
        local vm = minetest.get_voxel_manip()
        local minp, maxp = vm:read_from_map({x=pos.x-16, y=pos.y, z=pos.z-16}, {x=pos.x+16, y=pos.y+16, z=pos.z+16})
        local a = VoxelArea:new{MinEdge=minp, MaxEdge=maxp}
        local data = vm:get_data()
        default.grow_tree(data, a, pos, math.random(1, 4) == 1, math.random(1,100000))
        vm:set_data(data)
        vm:write_to_map(data)
        vm:update_map()
    end
})

-- ===================================================================
