-- ===================================================================

-- This parameter is a light level used in Outer Space realms. Due  to
-- an apparent bug in the MT mapgen API, this parameter should be less
-- than the maximum allowed level (15).

default.space_light_generic = 10

-- ===================================================================

local node_name
local use_spacesuit

-- ===================================================================

-- @@@
minetest.register_craftitem ("default:spacesuit", {
    description     = "Spacesuit Being Worn"       ,
    inventory_image = "default_spacesuit_item.png" ,

    on_use = function (itemstack, user, pointed_thing)
        if  itemstack:take_item() ~= nil then
            itemstack:add_item ("default:spacesuit_off")
        end
        return itemstack
    end ,
})

-- ===================================================================

-- @@@
minetest.register_craftitem ("default:spacesuit_off", {
    description     = "Spacesuit Taken Off"        ,
    inventory_image = "default_spacesuit_off.png"  ,

    on_use = function (itemstack, user, pointed_thing)
        local plname = user:get_player_name()

        local m =  miniskins.overlay_name [plname]
        if    m ~= nil then
            minetest.chat_send_player (plname,
                "You're already wearing armor [" .. m .. "]")
            return itemstack
        end

        if  itemstack:take_item() ~= nil then
            itemstack:add_item ("default:spacesuit")
        end

        return itemstack
    end ,
})

-- ===================================================================

-- @@@
minetest.register_craftitem ("default:crystal_armor", {
    description     = "Crystal Armor Being Worn"       ,
    inventory_image = "default_crystal_armor.png" ,

    on_use = function (itemstack, user, pointed_thing)
        if  itemstack:take_item() ~= nil then
            itemstack:add_item ("default:crystal_armor_off")
        end
        return itemstack
    end ,
})

-- ===================================================================

-- @@@
minetest.register_craftitem ("default:crystal_armor_off", {
    description     = "Crystal Armor Taken Off"    ,
    inventory_image = "default_crystal_armor_off.png"  ,

    on_use = function (itemstack, user, pointed_thing)
        local plname = user:get_player_name()

        local m =  miniskins.overlay_name [plname]
        if    m ~= nil then
            minetest.chat_send_player (plname,
                "You're already wearing armor [" .. m .. "]")
            return itemstack
        end

        if  itemstack:take_item() ~= nil then
            itemstack:add_item ("default:crystal_armor")
        end

        return itemstack
    end ,
})

-- ===================================================================

-- A  tick cycle length of greater than one doesn't  work well in this
-- context.

local tick       = 0
local tick_cycle = 1

-- @@@
minetest.register_globalstep (function (dtime)
    tick = tick + 1
    if tick < tick_cycle then return end
    tick = 0

    if  use_spacesuit == nil then
        use_spacesuit =  minetest.setting_get ("spacesuit")

        if ocutil.str_false (use_spacesuit) then
            use_spacesuit =  false
        end
        if  use_spacesuit == nil            then
            use_spacesuit =  true
        end
    end
    if not use_spacesuit then return end

    for _, player in ipairs (minetest.get_connected_players()) do
        local plname = player:get_player_name()

-- Check  to  see if the current player does or doesn't have an active
-- spacesuit and/or other armor in inventory.  Check for the spacesuit
-- last as that has highest priority.

        local has_spacesuit = false
        local oname         = nil

        if  player:get_inventory():contains_item ("main",
            "default:crystal_armor" ) then
              oname         = "crystal"
        end

        if  player:get_inventory():contains_item ("main",
            "default:spacesuit"     ) then
              has_spacesuit = true
              oname         = "spacesuit"
        end

        if ocutil.mod_exists ("coderskins" ) then
            local showing =  coderskins.overlay_name [plname]
            if    showing ~= oname then
                coderskins.change_skin (plname, oname)
            end
        end

        if ocutil.mod_exists ("miniskins"  ) then
            local showing =  miniskins.overlay_name [plname]
            if    showing ~= oname then
                miniskins.change_skin  (plname, oname)
            end
        end

        if math.random() < (0.1 * tick_cycle) then
            if has_spacesuit then
                if  player:get_breath() < 11 then
                    player:set_breath (11)
                end
            end
        end
                                -- Kludge to force bubbles update
        if math.random() > (1.00 - (tick_cycle / 100)) then
            local pos = player:getpos()
        end
    end
end)

-- ===================================================================

node_name = "default:vacuum"
--
minetest.register_node (node_name, {
    description         = "Vacuum"                    ,
    drawtype            = "airlike"                   ,
    paramtype           = "light"                     ,
    sunlight_propagates = true                        ,
    light_source        = default.space_light_generic ,
    walkable            = false                       ,
    pointable           = false                       ,
    diggable            = false                       ,
    buildable_to        = true                        ,
    drowning            = 1                           ,

    groups              = {
        vacuum=1,
        not_in_creative_inventory=1
    } ,
})

-- ===================================================================

local disable_vacuum  = false
if minetest.setting_getbool ("disable_vacuum") then
      disable_vacuum  = true
end

local c_air = minetest.get_content_id ("air")
if c_air == nil then ocutil.panic ("air undefined") end

if  disable_vacuum then
    default.c_vacuum = c_air
else
    default.c_vacuum = minetest.get_content_id ("default:vacuum")
end

-- ===================================================================
-- End of file.
