-- ===================================================================

ocsethome = {}
local                          disable_extra_homes =
    minetest.setting_getbool ("disable_extra_homes")

-- ===================================================================

local privdesc

if disable_extra_homes then
    privdesc = "For /sethome and /home"
else
    privdesc = "For /sethome /sethome2 /home /home2 etc."
end

minetest.register_privilege ("home", {
    description          = privdesc ,
    give_to_singleplayer = false    ,
})

local savedir = minetest.get_worldpath() .. "/sethome/"

-- This part requires "ocmkdir".
os.mkdir (savedir)

-- ===================================================================

ocsethome.sethome = function (name, d)

    local ds = ocsethome.disable_set      
    if ds ~= nil and ds then return end

    local player = minetest.env:get_player_by_name (name)
    local pos    = player:getpos()
    local pname  = player:get_player_name()

    d = "" .. d
    if d == "" then d = "1" end
    local fname  = savedir .. pname .. "_home" .. d
    local file   = io.open (fname, "w")

    if not file then
        minetest.chat_send_player (name,
"Error. Contact the server owner.")
        return
    end

    file:write (minetest.pos_to_string (pos))
    file:close()

-- Note: "ocsethome" calls "unified_inventory". However, "unified_
-- inventory" shouldn't be listed in the "ocsethome" "depends.txt"
-- file  (not even as an optional dependency.  Such an entry would
-- break things.

    if  d == "1" and
        _G ["unified_inventory"]   ~= nil and
        unified_inventory          ~= nil and
        unified_inventory.set_home ~= nil then

        ocsethome.disable_set = true
        unified_inventory.set_home (player, pos)
        ocsethome.disable_set = false
    end

    local            cmd = "/home" .. d
    if d == "1" then cmd = "/home or /home1" end

    minetest.chat_send_player (name,
"Home #" .. d .." is set. Use " .. cmd .. " to go there.")
end
 
-- ===================================================================

local register_cmd_sethome = function (txt_digit)

    if    txt_digit == nil then txt_digit = ""  end
    local num_digit =  txt_digit
    if    num_digit == ""  then num_digit = "1" end

    local param_sethome = {
        params      = ""                                 ,
        description = "Set home location #" .. num_digit ,
        privs       = { home=true }                      ,

        func = function (name, param)
            local use_digit = num_digit

            if param ~= nil and param ~= "" then
                local player = minetest.env:get_player_by_name (name)
                local pname  = player:get_player_name()
                param = tonumber (param)

                if param == nil then
                    minetest.chat_send_player (name,
"Usage: /sethome# or /sethome #")
                    return
                end

                if param < 1 or param > 4 then
                    minetest.chat_send_player (name,
"Error: You can only have homes numbered 1 to 4")
                    return
                end

                if txt_digit ~= "" then
                    minetest.chat_send_player (name,
"Error: /sethome# # doesn't make sense")
                    return
                end

                use_digit = param
            end

            ocsethome.sethome (name, use_digit)
        end
    }

    minetest.register_chatcommand ("sethome" .. txt_digit,
        param_sethome)

end

-- ===================================================================

register_cmd_sethome ("" )
register_cmd_sethome ("1")

if not disable_extra_homes then
    register_cmd_sethome ("2")
    register_cmd_sethome ("3")
    register_cmd_sethome ("4")
end

-- ===================================================================

ocsethome.home = function (name, d)

    local player = minetest.env:get_player_by_name (name)
    local pname  = player:get_player_name()

    d = "" .. d
    if d == "" then d = "1" end
    local fname = savedir .. pname .. "_home" .. d
    local file  = io.open (fname, "r")

    if not file then
        local cmd = "/home" .. d
        if d == "1" then cmd = "/sethome or /sethome1" end

        minetest.chat_send_player (name,
"You haven't set that location. Use " .. cmd .. " to do so.")
        return
    end

    local line = file:read ("*line")
    file:close()

    local pos = minetest.string_to_pos (string.sub (line,
        1, string.find (line, ")")))
    
    if not pos or type (pos) ~= "table" then
        minetest.chat_send_player (name,
"Error. Contact the server owner.")
        return
    end

    player:setpos (pos)
    minetest.chat_send_player (name, "Home sweet home #" .. d)
end

-- ===================================================================

local register_cmd_home = function (txt_digit)

    if    txt_digit == nil then txt_digit = ""  end
    local num_digit =  txt_digit
    if    num_digit == ""  then num_digit = "1" end

    local param_home = {
        params      = ""                                ,
        description = "Teleport to home #" .. num_digit ,
        privs       = { home=true }                     ,

        func = function (name, param)
            local use_digit = num_digit

            if param ~= nil and param ~= "" then
                local player = minetest.env:get_player_by_name (name)
                local pname  = player:get_player_name()
                param = tonumber (param)

                if param == nil then
                    minetest.chat_send_player (name,
"Usage: /home# or /home #")
                    return
                end

                if param < 1 or param > 4 then
                    minetest.chat_send_player (name,
"Error: You can only have homes numbered 1 to 4")
                    return
                end

                if txt_digit ~= "" then
                    minetest.chat_send_player (name,
"Error: /home# # doesn't make sense")
                    return
                end

                use_digit = param
            end

            ocsethome.home (name, use_digit)
        end
    }

    minetest.register_chatcommand ("home" .. txt_digit, param_home)
end

-- ===================================================================

register_cmd_home ("" )
register_cmd_home ("1")

if not disable_extra_homes then
    register_cmd_home ("2")
    register_cmd_home ("3")
    register_cmd_home ("4")
end

-- ===================================================================
-- Wrap it up.

minetest.log ("action", "mod ocsethome loaded")

-- ===================================================================
-- End of module.
