local modname = minetest.get_current_modname()
local modpath = minetest.get_modpath (modname)

local lightstone_colors = {
    "red"    , "green" , "blue" , "gray"    , "darkgray" , "yellow" ,
    "orange" , "white" , "pink" , "magenta" , "cyan"     , "violet" ,
}

-- ===================================================================

local function register_lightstone (color)

    local node     = "lightstone:" .. color
    local node_on  = "lightstone:" .. color .. "_on"
    local node_off = "lightstone:" .. color .. "_off"

    local img_on   = "lightstone_" .. color .. "_on.png"
    local img_off  = "lightstone_" .. color .. "_off.png"

    local groups    = {
        cracky=3, oddly_breakable_by_hand=3
    }

    local sounds    = default.sound_stone()

    local params_on = {
        tiles               = { img_on }                 ,
        inventory_image     = img_on                     ,
        description         = color .. " lightstone on"  ,
        sunlight_propagates = false                      ,
        is_ground_content   = false                      ,
        paramtype           = "light"                    ,
        groups              = groups                     ,
        sounds              = sounds                     ,
        light_source        = 14                         ,
    }

    local params_off = {
        tiles               = { img_off }                ,
        inventory_image     = img_off                    ,
        description         = color .. " lightstone off" ,
        sunlight_propagates = false                      ,
        is_ground_content   = false                      ,
        paramtype           = "light"                    ,
        groups              = groups                     ,
        sounds              = sounds                     ,
    }

    minetest.register_node (node_off , params_off )
    minetest.register_node (node_on  , params_on  )
    minetest.register_alias (node  , node_on)

if false then
    lightstone.register_stairs (color, nodename,
        params_on.groups             ,
        { img_on }                   ,
        color .. " lightstone stair" ,
        color .. " lightstone slab"  ,
        sounds)
end
end

-- ===================================================================

local on_lightstone_puncher = function (pos, node, puncher)
    local oldname = node.name
    local newname

    if not ocutil.starts_with (oldname, "lightstone:") then return end

    if ocutil.ends_with (oldname, "_off") then
        newname = oldname:gsub ("_off" , "_on"  )
    else
        newname = oldname:gsub ("_on"  , "_off" )
    end

    minetest.set_node (pos, { name=newname })
    nodeupdate (pos)
end

-- ===================================================================

minetest.register_on_punchnode (on_lightstone_puncher)

for  _,color in pairs (lightstone_colors) do
    register_lightstone (color)
end

-- dofile (modpath .. "/stairs.lua")
