ufos = {}

local modname = minetest.get_current_modname()
local modpath = minetest.get_modpath (modname)
local is_moontest = false

if minetest.setting_getbool ("enable_moon"     ) or
   minetest.setting_getbool ("enable_moontest" ) then
    is_moontest = true
end

if is_moontest then
    ufos.fuel = "moontest:lunarium_lump"
    ufos.UFO_SPEED      =  2
    ufos.UFO_TURN_SPEED =  4
    ufos.UFO_MAX_SPEED  = 20
    ufos.UFO_FUEL_USE   =  0.005
    ufos.UFO_MAX_FUEL   = 25
else
    ufos.fuel = "default:obsidian_shard"
    ufos.UFO_SPEED      =  1
    ufos.UFO_TURN_SPEED =  2
    ufos.UFO_MAX_SPEED  = 10
    ufos.UFO_FUEL_USE   =  0.01
    ufos.UFO_MAX_FUEL   = 50
end

local setfueltype = minetest.setting_get ("ufofueltype")

if ocutil.str_nonempty (setfueltype) and
   ocutil.node_exists (setfueltype) then
    ufos.fuel = setfueltype
end

local setmaxfuel = ocutil.numsetzero ("ufomaxfuel")

if setmaxfuel > 0 then
    if setmaxfuel <  10 then setmaxfuel =   5 end
    if setmaxfuel > 100 then setmaxfuel = 100 end
    ufos.UFO_MAX_FUEL = setmaxfuel
end

ufos.furnace_inactive_formspec =
    "size[8,6.6]"                                 ..
    "list[current_name;fuel;3.5,0;1,1;]"          ..
    "label[0,1.0;Fuel type: " .. ufos.fuel .. "]" ..
    "label[0,1.5;Park UFO near this or another charger]" ..
    "label[0,2.0;Press Special (usually E) or Back inside the UFO]" ..
    "list[current_player;main;0,2.6;8,4;]"

minetest.register_privilege ("ufo_admin",
    "For admin to remove abandoned ufos")

local MAXSP = ufos.UFO_MAX_SPEED

local floor_pos = function(pos)
    return {x=math.floor(pos.x),y=math.floor(pos.y),z=math.floor(pos.z)}
end

ufos.fuel_from_wear = function (wear)
    local fuel
    if wear == 0 then
        fuel = 0
    else
        fuel = (65535-(wear-1))*ufos.UFO_MAX_FUEL/65535
    end
    return fuel
end

ufos.wear_from_fuel = function(fuel)
    local wear = (ufos.UFO_MAX_FUEL-(fuel))*65535/ufos.UFO_MAX_FUEL+1
    if wear > 65535 then wear = 0 end
    return wear
end

ufos.get_fuel = function(self)
    return self.fuel
end

ufos.set_fuel = function (self, fuel,object)
    self.fuel = fuel
end

ufos.ufo_to_item = function(self)
    local wear = ufos.wear_from_fuel(ufos.get_fuel(self))
    return {name="ufos:ufo",wear=wear}
end

ufos.ufo_from_item = function(itemstack,placer,pointed_thing)
    -- set owner
    ufos.next_owner = placer:get_player_name()
    -- restore the fuel inside the item
    local wear = itemstack:get_wear()
    ufos.set_fuel(ufos.ufo,ufos.fuel_from_wear(wear))
    -- add the entity
    minetest.add_entity(pointed_thing.above, "ufos:ufo")
    -- remove the item
    itemstack:take_item()
    -- reset owner for next ufo
    ufos.next_owner = ""
end

ufos.check_owner = function(self, clicker)
    if minetest.check_player_privs(clicker:get_player_name(), {ufo_admin=true}) then
        return true
    end
    if self.owner_name ~= "" and clicker:get_player_name() ~= self.owner_name then
        minetest.chat_send_player(clicker:get_player_name(), "This UFO is owned by "..self.owner_name.." !")
        return false
    elseif self.owner_name == "" then
        minetest.chat_send_player(clicker:get_player_name(), "This UFO was not protected, you are now its owner !")
        self.owner_name = clicker:get_player_name()
    end
    return true
end

ufos.next_owner = ""

ufos.ufo = {
    physical = true,
    collisionbox = {-1.5,-.5,-1.5, 1.5,2,1.5},
    visual = "mesh",
    mesh = "ufo.x",
    textures = {"ufo_0.png"},

    driver = nil,
    owner_name = "",
    v = 0,
    fuel = 0,
    fueli = 0
}
function ufos.ufo:on_rightclick (clicker)
    if not clicker or not clicker:is_player() then
        return
    end
    if self.driver and clicker == self.driver then
        self.driver = nil
        clicker:set_detach()
    elseif not self.driver then
        if ufos.check_owner(self,clicker) then
            self.driver = clicker
            clicker:set_attach(self.object, "", {x=0,y=7.5,z=0}, {x=0,y=0,z=0})
        end
    end
end

function ufos.ufo:on_activate (staticdata, dtime_s)
    if ufos.next_owner ~= "" then
        self.owner_name = ufos.next_owner
        ufos.next_owner = ""
    else
        local data = staticdata:split(';')
        if data and data[1] and data[2] then
            self.owner_name = data[1]
            self.fuel = tonumber(data[2])
        end
    end
    self.object:set_armor_groups({immortal=1})
end

function ufos.ufo:on_punch (puncher, time_from_last_punch, tool_capabilities, direction)
    if puncher and puncher:is_player() then
        if ufos.check_owner(self,puncher) then
            puncher:get_inventory():add_item("main", ufos.ufo_to_item(self))
            self.object:remove()
        end
    end
end

function ufos.ufo:on_step (dtime)
    local fuel = ufos.get_fuel(self)
    if self.driver then
        local ctrl = self.driver:get_player_control()
        local vel = self.object:getvelocity()
        if fuel == nil then fuel = 0 end
        if fuel > 0 and ctrl.up then
            vel.x = vel.x + math.cos(self.object:getyaw()+math.pi/2)*ufos.UFO_SPEED
            vel.z = vel.z + math.sin(self.object:getyaw()+math.pi/2)*ufos.UFO_SPEED
            fuel = fuel - ufos.UFO_FUEL_USE
        else
            vel.x = vel.x*.99
            vel.z = vel.z*.99
        end
        if ctrl.down then
            vel.x = vel.x*.9
            vel.z = vel.z*.9
        end
        if fuel > 0 and ctrl.jump then
            vel.y = vel.y+ufos.UFO_SPEED
            fuel = fuel - ufos.UFO_FUEL_USE
        elseif ctrl.sneak then
            -- Note: Should go down even with empty fuel tank
            vel.y = vel.y-ufos.UFO_SPEED
            if fuel > 0 then fuel = fuel - ufos.UFO_FUEL_USE end
        else
            vel.y = vel.y * 0.9
        end

        if vel.x >  MAXSP then vel.x =  MAXSP end
        if vel.x < -MAXSP then vel.x = -MAXSP end
        if vel.y >  MAXSP then vel.y =  MAXSP end
        if vel.y < -MAXSP then vel.y = -MAXSP end
        if vel.z >  MAXSP then vel.z =  MAXSP end
        if vel.z < -MAXSP then vel.z = -MAXSP end

        self.object:setvelocity(vel)
        if ctrl.left then
            self.object:setyaw(self.object:getyaw()+math.pi/120*ufos.UFO_TURN_SPEED)
        end
        if ctrl.right then
            self.object:setyaw(self.object:getyaw()-math.pi/120*ufos.UFO_TURN_SPEED)
        end

        if ctrl.aux1 or ctrl.down then
            local playname = self.driver:get_player_name()
            if playname == nil then playname = "_Unknown_" end

            local pos   = self.object:getpos()
            local ucpos = minetest.find_node_near (pos,
                2.1, "ufos:furnace")

            if fuel >= 100 then
                minetest.chat_send_player (playname,
"UFO doesn't need fuel yet")
            end

            if ucpos == nil then
                minetest.chat_send_player (playname,
"You aren't parked near a UFO charger")
            else
                meta = minetest.env:get_meta (ucpos)
                if meta:get_int ("charge") < 1 then
                    minetest.chat_send_player (playname,
"UFO furnace is out of charge")
                end

                if fuel < 100 and meta:get_int ("charge") > 0 then
                    fuel = fuel + 1
                    meta:set_int ("charge", meta:get_int ("charge")-1)
                    meta:set_string("formspec", ufos.furnace_inactive_formspec
.. "label[0,0;Charge: "..meta:get_int("charge"))

                    minetest.chat_send_player (playname, "*ching*")
                end
            end

            if fuel <   0 then fuel =   0 end
            if fuel > 100 then fuel = 100 end

            minetest.chat_send_player (playname,
"UFO charge is at " .. fuel .. ".")
        end
    end

    if fuel < 0 then fuel = 0 end
    if fuel > ufos.UFO_MAX_FUEL then fuel = ufos.UFO_MAX_FUEL end
    fuel = math.floor (fuel + 0.5)

    local newlevel = math.floor (((fuel / ufos.UFO_MAX_FUEL) * 8) + 0.5)
    if newlevel > 8 then newlevel = 8 end

    if  self.fueli ~= newlevel then
        self.fueli =  newlevel
        self.textures = { "ufo_" .. self.fueli .. ".png" }
        self.object:set_properties (self)
    end

    ufos.set_fuel (self, fuel)
end

function ufos.ufo:get_staticdata()
    return self.owner_name..";"..tostring(self.fuel)
end

minetest.register_entity("ufos:ufo", ufos.ufo)


minetest.register_tool("ufos:ufo", {
    description = "ufo",
    inventory_image = "ufos_inventory.png",
    wield_image = "ufos_inventory.png",
    tool_capabilities = {load=0,max_drop_level=0, groupcaps={fleshy={times={}, uses=100, maxlevel=0}}},

    on_place = function(itemstack, placer, pointed_thing)
        if pointed_thing.type ~= "node" then
            return
        end

        -- Call on_rightclick if the pointed node defines it
        if placer and not placer:get_player_control().sneak then
            local n = minetest.get_node(pointed_thing.under)
            local nn = n.name
            if minetest.registered_nodes[nn] and minetest.registered_nodes[nn].on_rightclick then
                return minetest.registered_nodes[nn].on_rightclick(pointed_thing.under, n, placer, itemstack) or itemstack
            end
        end

        ufos.ufo_from_item(itemstack,placer,pointed_thing)
        return itemstack
    end,
})

-- ufos box kept for compatibility only
minetest.register_node("ufos:box", {
    description = "UFO BOX (you hacker you!)",
    tiles = {"ufos_box.png"},
    groups = {not_in_creative_inventory=1},
    on_rightclick = function(pos, node, clicker, itemstack)
        meta = minetest.get_meta(pos)
        if meta:get_string("owner") == clicker:get_player_name() then
            -- set owner
            ufos.next_owner = meta:get_string("owner")
            -- restore the fuel inside the node
            ufos.set_fuel(ufos.ufo,meta:get_int("fuel"))
            -- add the entity
            e = minetest.add_entity(pos, "ufos:ufo")
            -- remove the node
            minetest.remove_node(pos)
            -- reset owner for next ufo
            ufos.next_owner = ""
        end
    end,
})

if is_moontest then
minetest.register_craft( {
    output = 'ufos:ufo',
    recipe = {
        { "", "default:glass", ""},
        { "moontest:lunarium_lump", "", "moontest:lunarium_lump"},
        { "default:steelblock", "moontest:lunarium_ingot", "default:steelblock"},
    },
})
else
minetest.register_craft( {
	output = 'ufos:ufo',
	recipe = {
		{ "", "default:glass", ""},
		{ "default:mese_crystal_fragment", "", "default:mese_crystal_fragment"},
		{ "default:steelblock", "default:mese", "default:steelblock"},
	},
})
end

dofile (modpath .. "/furnace.lua")
